class KVNode(object):
    def __init__(self, key, value):
        self.key = key
        self.value = value
        self.next = None

    def __repr__(self):
        """
        >>> d = KVNode('a', 1)
        >>> d
        KVNode('a', 1)
        """
        return 'KVNode({}, {})'.format(repr(self.key), repr(self.value))


class SequentialSearchKVStore(object):

    def __init__(self, d=None):
        """
        >>> d = SequentialSearchKVStore({'b': 2, 'c': 3, 'a': 1})
        >>> d
        SequentialSearchKVStore({'b': 2, 'c': 3, 'a': 1})
        """
        self.n = 0
        self.head = None
        if d:
            store = SequentialSearchKVStore()
            for k, v in d.items():
                store[k] = v
            self.n = store.n
            self.head = store.head

    def __len__(self):
        """
        >>> d = SequentialSearchKVStore()
        >>> len(d)
        0
        >>> d['a'] = 1
        >>> len(d)
        1
        >>> d['b'] = 2
        >>> len(d)
        2
        >>> d['a'] = 3
        >>> len(d)
        2
        """
        return self.n

    def __contains__(self, key):
        """
        >>> d = SequentialSearchKVStore()
        >>> 'a' in d
        False
        >>> d['a'] = 1
        >>> 'a' in d
        True
        """
        return self._get_node(key) is not None

    def __getitem__(self, key):
        """
        >>> d = SequentialSearchKVStore()
        >>> d['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        >>> d['a'] = 1
        >>> d['a']
        1
        >>> d['a'] = 2
        >>> d['a']
        2
        """
        n = self._get_node(key)
        if n is None:
            raise KeyError(key)
        return n.value

    def __setitem__(self, key, value):
        """
        >>> d = SequentialSearchKVStore()
        >>> d['a'] = 1
        """
        n = self._get_node(key)
        if n:
            n.value = value
            return
        node = KVNode(key, value)
        node.next = self.head
        self.head = node
        self.n += 1

    def __delitem__(self, key):
        """
        >>> d = SequentialSearchKVStore()
        >>> d['a'] = 1
        >>> 'a' in d
        True
        >>> len(d)
        1
        >>> del d['a']
        >>> 'a' in d
        False
        >>> len(d)
        0
        >>> del d['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        """
        n = self.n
        self.head = self._delete_node(self.head, key)
        if n == self.n:
            raise KeyError(key)

    def __iter__(self):
        """
        >>> d = SequentialSearchKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> d['c'] = 3
        >>> for i in d:
        ...     print i
        ...
        c
        b
        a
        """
        n = self.head
        while n:
            yield n.key
            n = n.next

    def items(self):
        """
        >>> d = SequentialSearchKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> d['c'] = 3
        >>> for k, v in d.items():
        ...     print k, v
        ...
        c 3
        b 2
        a 1
        """
        n = self.head
        while n:
            yield n.key, n.value
            n = n.next

    def __repr__(self):
        """
        >>> d = SequentialSearchKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> d['c'] = 3
        >>> d
        SequentialSearchKVStore({'c': 3, 'b': 2, 'a': 1})
        """
        s = ', '.join('{}: {}'.format(
            repr(k), repr(v))
            for k, v in self.items()
        )
        return 'SequentialSearchKVStore({%s})' % s

    def _get_node(self, key):
        n = self.head
        for _ in range(len(self)):
            if n.key == key:
                return n
            n = n.next

    def _delete_node(self, node, key):
        if node is None:
            return
        if node.key == key:
            self.n -= 1
            return node.next
        node.next = self._delete_node(node.next, key)
        return node



from lib_collection.node import Node


class SimpleLinkedQueue(object):

    def __init__(self, lst=None):
        """
        >>> q = SimpleLinkedQueue(['a', 2, 'c'])
        >>> q.top
        'a'
        >>> q.dequeue()
        'a'
        >>> q.dequeue()
        2
        >>> len(q)
        1
        >>> q.enqueue(4)
        >>> q.enqueue('e')
        >>> q
        SimpleLinkedQueue(['c', 4, 'e'])
        """
        self.n = 0
        self.head = Node(None)

        if lst:
            queue = SimpleLinkedQueue()
            for e in lst:
                queue.enqueue(e)
            self.n = queue.n
            self.head = queue.head

    def __len__(self):
        """
        >>> q = SimpleLinkedQueue()
        >>> len(q)
        0
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.enqueue('c')
        >>> len(q)
        3
        >>> q.dequeue()
        'a'
        >>> len(q)
        2
        """
        return self.n

    def __contains__(self, i):
        """
        >>> q = SimpleLinkedQueue()
        >>> 'a' in q
        False
        >>> q.enqueue('a')
        >>> 'a' in q
        True
        """
        for j in self:
            if i == j:
                return True
        return False

    def __iter__(self):
        """
        >>> q = SimpleLinkedQueue()
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.enqueue('c')
        >>> for i in q:
        ...     print i
        ...
        a
        b
        c
        """
        n = self.head
        while n.next is not None:
            n = n.next
            yield n.v

    def __repr__(self):
        """
        >>> q = SimpleLinkedQueue()
        >>> q.enqueue('a')
        >>> q.enqueue(2)
        >>> q.enqueue('c')
        >>> q
        SimpleLinkedQueue(['a', 2, 'c'])
        >>> print q
        SimpleLinkedQueue(['a', 2, 'c'])
        """
        return 'SimpleLinkedQueue([{}])'.format(', '.join(repr(i) for i in self))

    def enqueue(self, i):
        """
        >>> q = SimpleLinkedQueue()
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        """
        node = Node(i)
        n = self.head
        while n.next is not None:
            n = n.next
        n.next = node
        self.n += 1

    def dequeue(self):
        """
        >>> q = SimpleLinkedQueue()
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.enqueue('c')
        >>> q.dequeue()
        'a'
        >>> q.dequeue()
        'b'
        >>> q.dequeue()
        'c'
        >>> q.dequeue()
        Traceback (most recent call last):
            ...
        IndexError: dequeue from empty queue
        """
        if len(self) == 0:
            raise IndexError('dequeue from empty queue')
        res = self.head.next.v
        self.head.next = self.head.next.next
        self.n -= 1
        return res

    @property
    def top(self):
        """
        >>> q = SimpleLinkedQueue()
        >>> q.top
        Traceback (most recent call last):
            ...
        IndexError: top from empty queue
        >>> q.enqueue('a')
        >>> q.top
        'a'
        >>> q.enqueue('b')
        >>> q.top
        'a'
        """
        if len(self) == 0:
            raise IndexError('top from empty queue')
        return self.head.next.v



class NoneNode(object):
    def __repr__(self):
        return 'NoneNode()'


none = NoneNode()

class BinarySearchKVStore(object):

    def __init__(self, capacity=2):
        self.n = 0
        self.keys = [none] * capacity
        self.values = [none] * capacity
        self.capacity = capacity

    def __len__(self):
        """
        >>> d = BinarySearchKVStore()
        >>> len(d)
        0
        >>> d['a'] = 1
        >>> len(d)
        1
        >>> d['b'] = 2
        >>> d['c'] = 3
        >>> len(d)
        3
        >>> del d['a']
        >>> len(d)
        2
        """
        return self.n

    def __contains__(self, key):
        """
        >>> d = BinarySearchKVStore()
        >>> 'a' in d
        False
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> 'a' in d
        True
        """
        i = self.index(key)
        if i < len(self) and self.keys[i] == key:
            return True
        return False

    def __setitem__(self, key, value):
        """
        >>> # 1. test key in keys
        >>> d = BinarySearchKVStore()
        >>> d.keys = [1, 2]
        >>> d.values = ['a', 'b']
        >>> d.n = 2
        >>> d[1] = 'c'
        >>> d.values = ['c', 2]
        >>> # 2. test full and resize
        >>> d = BinarySearchKVStore()
        >>> d.keys = [1, 2]
        >>> d.values = ['a', 'b']
        >>> d.n = 2
        >>> d[3] = 'c'
        >>> d.keys
        [1, 2, 3, NoneNode()]
        >>> d.capacity
        4
        >>> # 3. test insert new key in the end
        >>> d = BinarySearchKVStore()
        >>> d.keys = [1, 2]
        >>> d.values = ['a', 'b']
        >>> d.n = 2
        >>> d[3] = 'c'
        >>> d.keys
        [1, 2, 3, NoneNode()]
        >>> d.values
        ['a', 'b', 'c', NoneNode()]
        >>> # 4. test insert new key in the middle
        >>> d = BinarySearchKVStore()
        >>> d.keys = [1, 3]
        >>> d.values = ['a', 'c']
        >>> d.n = 2
        >>> d[2] = 'b'
        >>> d.keys
        [1, 2, 3, NoneNode()]
        >>> d.values
        ['a', 'b', 'c', NoneNode()]
        >>> # 5. test insert new key in the head
        >>> d = BinarySearchKVStore()
        >>> d.keys = [2, 3]
        >>> d.values = ['b', 'c']
        >>> d.n = 2
        >>> d[1] = 'a'
        >>> d.keys
        [1, 2, 3, NoneNode()]
        >>> d.values
        ['a', 'b', 'c', NoneNode()]
        """
        i = self.index(key)

        # key already in keys
        if i < len(self) and self.keys[i] == key:
            self.values[i] = value
            return

        # keys are full
        if len(self) == self.capacity:
            self._resize(self.capacity*2)

        # make space for the new key
        for j in range(len(self), i, -1):
            self.keys[j] = self.keys[j-1]
            self.values[j] = self.values[j-1]

        # insert new key
        self.keys[i] = key
        self.values[i] = value
        self.n += 1

    def __getitem__(self, key):
        """
        >>> d = BinarySearchKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> d['a']
        1
        >>> d['b']
        2
        >>> d['c']
        Traceback (most recent call last):
            ...
        KeyError: 'c'
        """
        i = self.index(key)
        if i < len(self) and self.keys[i] == key:
            return self.values[i]
        raise KeyError(key)

    def __delitem__(self, key):
        """
        >>> # 1. test delete item that not exists
        >>> d = BinarySearchKVStore()
        >>> del d['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        >>> # 2. test delete item that exists
        >>> d = BinarySearchKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> d['c'] = 3
        >>> d.keys
        ['a', 'b', 'c', NoneNode()]
        >>> del d['a']
        >>> d.keys
        ['b', 'c', NoneNode(), NoneNode()]
        >>> # 3. test delete item and resize down
        >>> d = BinarySearchKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> d['c'] = 3
        >>> d['d'] = 4
        >>> d['e'] = 5
        >>> del d['a']
        >>> del d['b']
        >>> d.capacity
        8
        >>> del d['c']
        >>> d.capacity
        4
        >>> del d['d']
        >>> d.capacity
        2
        >>> del d['e']
        >>> d.capacity
        2
        """
        i = self.index(key)
        if i == len(self) or self.keys[i] != key:
            raise KeyError(key)

        for j in range(i, len(self)-1):
            self.keys[j] = self.keys[j+1]
            self.values[j] = self.values[j+1]
        self.n -= 1
        self.keys[self.n] = none
        self.values[self.n] = none

        if len(self) * 4 <= self.capacity and len(self):
            self._resize(self.capacity//2)

    def __repr__(self):
        """
        >>> d = BinarySearchKVStore()
        >>> d['b'] = 2
        >>> d['a'] = 3
        >>> d['c'] = 1
        >>> d
        BinarySearchKVStore({'a': 3, 'b': 2, 'c': 1})
        """
        s = ', '.join('{}: {}'.format(repr(k), v) for k, v in self.items())
        return 'BinarySearchKVStore({%s})' % s

    def __iter__(self):
        """
        >>> d = BinarySearchKVStore()
        >>> d['b'] = 2
        >>> d['a'] = 3
        >>> d['c'] = 1
        >>> for i in d:
        ...     print i
        ...
        a
        b
        c
        """
        for i in range(len(self)):
            yield self.keys[i]

    def items(self):
        """
        >>> d = BinarySearchKVStore()
        >>> d['b'] = 2
        >>> d['a'] = 3
        >>> d['c'] = 1
        >>> for k, v in d.items():
        ...     print k, v
        ...
        a 3
        b 2
        c 1
        """
        for i in range(len(self)):
            yield self.keys[i], self.values[i]

    def index(self, key):
        """
        >>> d = BinarySearchKVStore()
        >>> d.n = 3
        >>> d.keys = [1, 2, 3]
        >>> d.index(2)
        1
        >>> d.index(1.5)
        1
        >>> d.index(2.5)
        2
        """
        lo = 0
        hi = self.n - 1
        while lo <= hi:
            mid = (lo + hi) // 2
            if key == self.keys[mid]:
                return mid
            elif key < self.keys[mid]:
                hi = mid - 1
            else:
                lo = mid + 1
        return lo

    @property
    def min(self):
        """
        >>> # 1. test get min from empty raise underflow error
        >>> d = BinarySearchKVStore()
        >>> d.min
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. test get min
        >>> d['b'] = 2
        >>> d['a'] = 3
        >>> d.min
        'a'
        """
        if len(self) == 0:
            raise IndexError('underflow')
        return self.keys[0]

    @property
    def max(self):
        """
        >>> # 1. test get max from empty raise underflow error
        >>> d = BinarySearchKVStore()
        >>> d.max
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. test get max
        >>> d['a'] = 3
        >>> d['c'] = 1
        >>> d['b'] = 2
        >>> d.max
        'c'
        """
        if len(self) == 0:
            raise IndexError('underflow')
        return self.keys[self.n-1]

    def del_max(self):
        """
        >>> # 1. test del_max from empty kvstore raise index error
        >>> d = BinarySearchKVStore()
        >>> d.del_max()
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. test del_max
        >>> d['b'] = 2
        >>> d['c'] = 1
        >>> d['a'] = 3
        >>> d.del_max()
        >>> d.max
        'b'
        >>> len(d)
        2
        >>> d.del_max()
        >>> d.max
        'a'
        >>> len(d)
        1
        """
        if len(self) == 0:
            raise IndexError('underflow')
        del self[self.max]

    def del_min(self):
        """
        >>> # 1. test del_max from empty kvstore raise index error
        >>> d = BinarySearchKVStore()
        >>> d.del_min()
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. test del_max
        >>> d = BinarySearchKVStore()
        >>> d['b'] = 2
        >>> d['c'] = 1
        >>> d['a'] = 3
        >>> d.del_min()
        >>> d.min
        'b'
        >>> len(d)
        2
        >>> d.del_min()
        >>> d.min
        'c'
        >>> len(d)
        1
        """
        if len(self) == 0:
            raise IndexError('underflow')
        del self[self.min]

    def get_key_at_index(self, i):
        """
        >>> d = BinarySearchKVStore()
        >>> d['b'] = 2
        >>> d['c'] = 1
        >>> d['a'] = 3
        >>> d.get_key_at_index(0)
        'a'
        >>> d.get_key_at_index(1)
        'b'
        >>> d.get_key_at_index(2)
        'c'
        >>> d.get_key_at_index(3)
        Traceback (most recent call last):
            ...
        IndexError: 3
        """
        if not 0 <= i < len(self):
            raise IndexError(i)
        return self.keys[i]

    def floor(self, key):
        """
        Return the largest key in kvstore that is less than or equal to key
        >>> d = BinarySearchKVStore()
        >>> d[1] = 'a'
        >>> d[2] = 'b'
        >>> d[3] = 'c'
        >>> d.floor(2)
        2
        >>> d.floor(1.1)
        1
        >>> d.floor(4)
        3
        >>> d.floor(0.9)

        """
        i = self.index(key)
        if i < len(self) and self.keys[i] == key:
            return self.keys[i]
        elif i == 0:
            return  # so key in this kvstore cannot be None
        return self.keys[i-1]

    def ceiling(self, key):
        """
        Return the smallest key in kvstore that is greater than or equal to key
        >>> d = BinarySearchKVStore()
        >>> d[1] = 'a'
        >>> d[2] = 'b'
        >>> d[3] = 'c'
        >>> d.ceiling(1.5)
        2
        >>> d.ceiling(0.5)
        1
        >>> d.ceiling(2)
        2
        >>> d.ceiling(5)

        """
        i = self.index(key)
        if i == len(self):
            return
        return self.keys[i]

    def size(self, key1, key2):
        """
        return the numbers of keys in this kvstore that min(key1, key2) <= key <= max(key1, key2)
        >>> d = BinarySearchKVStore()
        >>> d[1] = 'a'
        >>> d[2] = 'b'
        >>> d[3] = 'c'
        >>> d.size(1.5, 2.5)
        1
        >>> d.size(1.5, 3)
        2
        >>> d.size(1, 3)
        3
        """
        lo = min(key1, key2)
        hi = max(key1, key2)
        i = self.index(lo)
        j = self.index(hi)
        if hi in self:
            return j - i + 1
        return j - i

    def _resize(self, capacity):
        """
        >>> # 1. test resize up
        >>> d = BinarySearchKVStore()
        >>> d.n = 2
        >>> d.keys = [1, 2]
        >>> d.values = ['a', 'b']
        >>> d._resize(4)
        >>> d.keys
        [1, 2, NoneNode(), NoneNode()]
        >>> d.values
        ['a', 'b', NoneNode(), NoneNode()]
        >>> # 2. test resize Down
        >>> d = BinarySearchKVStore()
        >>> d.n = 2
        >>> d.keys = [1, 2, NoneNode(), NoneNode(), NoneNode(), NoneNode(), NoneNode(), NoneNode()]
        >>> d.values = ['a', 'b', NoneNode(), NoneNode(), NoneNode(), NoneNode(), NoneNode(), NoneNode()]
        >>> d._resize(2)
        >>> d.keys
        [1, 2]
        >>> d.values
        ['a', 'b']
        """
        keys = [none] * capacity
        values = [none] * capacity
        for i in range(len(self)):
            keys[i] = self.keys[i]
            values[i] = self.values[i]

        self.keys = keys
        self.values = values
        self.capacity = capacity



def sort(lst):
    """
    >>> lst = [5, 4, 3, 2, 1]
    >>> sort(lst)
    >>> lst
    [1, 2, 3, 4, 5]
    """
    _sort(lst, 0, len(lst)-1)


def _sort(lst, lo, hi):
    if lo >= hi:
        return

    p = partition(lst, lo, hi)
    _sort(lst, lo, p-1)
    _sort(lst, p+1, hi)


def partition(lst, lo, hi):
    value = lst[lo]

    left = lo + 1
    right = hi
    done = False

    while not done:
        while left <= right and lst[left] <= value:
            left += 1
        while left <= right and lst[right] >= value:
            right -= 1
        if right < left:
            done = True
        else:
            lst[left], lst[right] = lst[right], lst[left]
    lst[right], lst[lo] = lst[lo], lst[right]
    return right



class FixedCapacityArrayQueue(object):

    def __init__(self, lst=None, capacity=5):
        """
        >>> q = FixedCapacityArrayQueue(['a', 'b', 'c'])
        >>> q
        FixedCapacityArrayQueue(['a', 'b', 'c'])
        """
        self.capacity = capacity
        self.lst = [None] * capacity
        self.n = 0
        self.head = 0
        self.tail = 0

        if lst:
            q = FixedCapacityArrayQueue(capacity=capacity)
            for i in lst:
                q.enqueue(i)
            self.lst = q.lst
            self.n = q.n
            self.head = q.head
            self.tail = q.tail

    def __len__(self):
        """
        >>> q = FixedCapacityArrayQueue(capacity=4)
        >>> len(q)
        0
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.enqueue('c')
        >>> q.dequeue()
        'a'
        >>> q.dequeue()
        'b'
        >>> len(q)
        1
        >>> q.enqueue('d')
        >>> q.enqueue('e')
        >>> q.enqueue('f')
        >>> len(q)
        4
        """
        return self.n

    def __contains__(self, i):
        """
        >>> q = FixedCapacityArrayQueue(capacity=4)
        >>> 'a' in q
        False
        >>> q.enqueue('a')
        >>> 'a' in q
        True
        """
        for j in self:
            if i == j:
                return True
        return False

    def __iter__(self):
        """
        >>> q = FixedCapacityArrayQueue(capacity=4)
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.enqueue('c')
        >>> q.dequeue()
        'a'
        >>> q.dequeue()
        'b'
        >>> q.enqueue('d')
        >>> q.enqueue('e')
        >>> q.enqueue('f')
        >>> q.lst
        ['e', 'f', 'c', 'd']
        >>> for i in q:
        ...     print i
        ...
        c
        d
        e
        f
        """
        n = self.head
        for i in range(len(self)):
            yield self.lst[n]
            n = (n+1) % self.capacity

    def __repr__(self):
        """
        >>> q = FixedCapacityArrayQueue(capacity=4)
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.enqueue('c')
        >>> q.dequeue()
        'a'
        >>> q.dequeue()
        'b'
        >>> q.enqueue('d')
        >>> q.enqueue('e')
        >>> q.enqueue('f')
        >>> q.lst
        ['e', 'f', 'c', 'd']
        >>> q
        FixedCapacityArrayQueue(['c', 'd', 'e', 'f'])
        """
        return 'FixedCapacityArrayQueue([{}])'.format(', '.join(repr(i) for i in self))

    def enqueue(self, i):
        """
        >>> q = FixedCapacityArrayQueue(capacity=0)
        >>> q.enqueue('a')
        Traceback (most recent call last):
            ...
        IndexError: queue overflow
        >>> q = FixedCapacityArrayQueue(capacity=2)
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.lst
        ['a', 'b']
        >>> q.tail
        0
        """
        if len(self) == self.capacity:
            raise IndexError('queue overflow')

        self.lst[self.tail]= i
        self.tail = (self.tail + 1) % self.capacity
        self.n += 1

    def dequeue(self):
        """
        >>> q = FixedCapacityArrayQueue(capacity=0)
        >>> q.dequeue()
        Traceback (most recent call last):
            ...
        IndexError: queue underflow
        >>> q = FixedCapacityArrayQueue(capacity=2)
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.dequeue()
        'a'
        >>> q.enqueue('c')
        >>> q.dequeue()
        'b'
        >>> q.enqueue('d')
        >>> q.dequeue()
        'c'
        """
        if len(self) == 0:
            raise IndexError('queue underflow')
        res = self.lst[self.head]
        self.head = (self.head + 1) % self.capacity
        self.n -= 1
        return res

    @property
    def top(self):
        """
        >>> q = FixedCapacityArrayQueue(capacity=3)
        >>> q.top
        Traceback (most recent call last):
            ...
        IndexError: queue underflow
        >>> q.enqueue('a')
        >>> q.top
        'a'
        >>> q.enqueue('b')
        >>> q.top
        'a'
        """
        if len(self) == 0:
            raise IndexError('queue underflow')
        return self.lst[self.head]



def sort(lst):
    """
    >>> lst = [6, 5, 4, 3, 2]
    >>> sort(lst)
    >>> lst
    [2, 3, 4, 5, 6]
    """
    n = len(lst)
    done = False
    round = n - 1
    while not done and round:
        done = True
        for i in range(round):
            if lst[i] > lst[i+1]:
                lst[i], lst[i+1] = lst[i+1], lst[i]
                done = False
                # if there is no swap, the list is sorted already
        round -= 1



from lib_collection.node import Node


class LinkedStack(object):

    def __init__(self, lst=None):
        """
        >>> stack = LinkedStack(["a", 1, "b"])
        >>> stack.pop()
        'a'
        >>> stack.push('c')
        >>> stack
        LinkedStack(['c', 1, 'b'])
        """
        self.n = 0
        self.head = Node(None)

        if lst:
            stack = LinkedStack()
            for e in lst[::-1]:
                stack.push(e)
            self.n = stack.n
            self.head = stack.head

    def __len__(self):
        """
        >>> s = LinkedStack()
        >>> len(s)
        0
        >>> s.push('a')
        >>> s.push('b')
        >>> len(s)
        2
        >>> s.pop()
        'b'
        >>> len(s)
        1
        """
        return self.n

    def __contains__(self, i):
        """
        >>> s = LinkedStack()
        >>> s.push('a')
        >>> s.push('b')
        >>> 'a' in s
        True
        >>> 'c' in s
        False
        """
        for j in self:
            if j == i:
                return True
        return False

    def __iter__(self):
        """
        >>> s = LinkedStack()
        >>> s.push('a')
        >>> s.push('b')
        >>> s.push('c')
        >>> for i in s:
        ...     print i
        ...
        c
        b
        a
        """
        n = self.head.next
        while n:
            yield n.v
            n = n.next

    def __repr__(self):
        """
        >>> s = LinkedStack()
        >>> s.push('a')
        >>> s.push(2)
        >>> s.push('c')
        >>> s
        LinkedStack(['c', 2, 'a'])
        >>> print s
        LinkedStack(['c', 2, 'a'])
        """
        return 'LinkedStack([{}])'.format(', '.join(repr(i) for i in self))

    def push(self, i):
        """
        >>> s = LinkedStack()
        >>> s.push('a')
        """
        n = Node(i)
        n.next = self.head.next
        self.head.next = n
        self.n += 1

    def pop(self):
        """
        >>> s = LinkedStack()
        >>> s.push('a')
        >>> s.push('b')
        >>> s.pop()
        'b'
        >>> s.pop()
        'a'
        >>> s.pop()
        Traceback (most recent call last):
            ...
        IndexError: pop from empty stack
        """
        if len(self) == 0:
            raise IndexError('pop from empty stack')
        res = self.head.next.v
        self.head.next = self.head.next.next
        self.n -= 1
        return res

    @property
    def top(self):
        """
        >>> s = LinkedStack()
        >>> s.top
        Traceback (most recent call last):
            ...
        IndexError: pop from empty stack
        >>> s.push('a')
        >>> s.top
        'a'
        """
        if len(self) == 0:
            raise IndexError('pop from empty stack')
        return self.head.next.v



class FixedSizeMinPriorityQueue(object):

    def __init__(self, capacity=10):
        self.keys = [None] * capacity
        self.n = 0

    def __len__(self):
        """
        >>> q = FixedSizeMinPriorityQueue()
        >>> len(q)
        0
        >>> q.insert(3)
        >>> q.insert(2)
        >>> q.insert(1)
        >>> len(q)
        3
        >>> q.pop()
        1
        >>> len(q)
        2
        """
        return self.n

    @property
    def min(self):
        """
        >>> # 1. Test underflow
        >>> q = FixedSizeMinPriorityQueue()
        >>> q.min
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. Test min
        >>> q.insert(3)
        >>> q.min
        3
        >>> q.insert(2)
        >>> q.min
        2
        >>> q.insert(1)
        >>> q.min
        1
        >>> q.insert(4)
        >>> q.min
        1
        """
        if self.n == 0:
            raise IndexError('underflow')
        return self.keys[1]

    def pop(self):
        """
        >>> q = FixedSizeMinPriorityQueue()
        >>> q.insert(5)
        >>> q.insert(4)
        >>> q.insert(3)
        >>> q.insert(2)
        >>> q.insert(1)
        >>> q.pop()
        1
        >>> q.pop()
        2
        >>> q.pop()
        3
        >>> q.pop()
        4
        >>> q.pop()
        5
        """
        keys = self.keys
        res = keys[1]
        keys[1], keys[self.n] = keys[self.n], keys[1]
        keys[self.n] = None
        self.n -= 1
        self._sink(1)
        return res

    def insert(self, i):
        """
        >>> q = FixedSizeMinPriorityQueue(capacity=4)
        >>> q.insert(3)
        >>> q.insert(2)
        >>> q.insert(1)
        >>> q.keys
        [None, 1, 3, 2]
        """
        self.n += 1
        self.keys[self.n] = i
        self._swim(self.n)

    def _swim(self, n):
        """
        >>> q = FixedSizeMinPriorityQueue()
        >>> q.keys = [None, 2, 3, None, 1]
        >>> q.n = 4
        >>> q._swim(4)
        >>> q.keys
        [None, 1, 2, None, 3]
        """
        keys = self.keys
        while n > 1 and keys[n/2] > keys[n]:
            keys[n/2], keys[n] = keys[n], keys[n/2]
            n /= 2

    def _sink(self, n):
        """
        >>> q = FixedSizeMinPriorityQueue()
        >>> q.keys = [None, 3, 1, 2, None]
        >>> q.n = 3
        >>> q._sink(1)
        >>> q.keys
        [None, 1, 3, 2, None]
        >>> q = FixedSizeMinPriorityQueue()
        >>> q.keys = [None, 3, 2, 1, None]
        >>> q.n = 3
        >>> q._sink(1)
        >>> q.keys
        [None, 1, 2, 3, None]
        """
        keys = self.keys
        while n * 2 <= self.n:
            i = n * 2
            if i < self.n and keys[i+1] < keys[i]:
                i += 1
            if keys[n] <= keys[i]:
                break
            keys[n], keys[i] = keys[i], keys[n]
            n = i



from lib_collection.queue.linked_queue import LinkedQueue


class Node(object):
    def __init__(self, k, v):
        self.k = k
        self.v = v
        self.left = None
        self.right = None
        self.size = 1

    def __repr__(self):
        """
        >>> n = Node(1, 'a')
        >>> n
        Node(1, 'a')
        """
        return 'Node({}, {})'.format(repr(self.k), repr(self.v))


class BSTKVStore(object):

    def __init__(self):
        self.root = None
        self.n = 0

    def __setitem__(self, k, v):
        """
        >>> s = BSTKVStore()
        >>> s[2] = 'b'
        >>> s.root
        Node(2, 'b')
        >>> s[1] = 'a'
        >>> s.root.left
        Node(1, 'a')
        >>> s[3] = 'c'
        >>> s.root.right
        Node(3, 'c')
        """
        self.root = self._put_to_node(self.root, k, v)

    def __getitem__(self, k):
        """
        >>> # 1. test KeyError
        >>> s = BSTKVStore()
        >>> s[5]
        Traceback (most recent call last):
            ...
        KeyError: 5
        >>> # 2. test found
        >>> s[1] = 'a'
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> s[3]
        'c'
        >>> s[2]
        'b'
        >>> s[1]
        'a'
        """
        n = self._get_node(self.root, k)
        if n is None:
            raise KeyError(k)
        return n.v

    def __len__(self):
        """
        >>> s = BSTKVStore()
        >>> len(s)
        0
        >>> s[1] = 'a'
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> len(s)
        3
        >>> del s[3]
        >>> len(s)
        2
        """
        return self._get_size(self.root)

    def _put_to_node(self, node, k, v):
        if node is None:
            return Node(k, v)

        if node.k == k:
            node.v = v
        elif k < node.k:
            node.left = self._put_to_node(node.left, k, v)
        else:
            node.right = self._put_to_node(node.right, k, v)

        node.size = 1 + self._get_size(node.left) + self._get_size(node.right)
        return node

    def _get_size(self, node):
        if node is None:
            return 0
        return node.size

    def _get_node(self, node, k):
        """
        >>> # 1. test the node passed in is None
        >>> s = BSTKVStore()
        >>> n = s._get_node(None, 'k')
        >>> n

        >>> # 2. test k equal
        >>> s = BSTKVStore()
        >>> n = Node(1, 'a')
        >>> res = s._get_node(n, 1)
        >>> res is n
        True
        >>> # 3. test k less than k of node and found
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'a')
        >>> b.left = a
        >>> res = s._get_node(b, 1)
        >>> res is a
        True
        >>> # 4. test k less than k of node and not found
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'a')
        >>> b.left = a
        >>> res = s._get_node(b, 1.5)
        >>> res is None
        True
        >>> # 5. test k greater than k of node and found
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.right = c
        >>> res = s._get_node(b, 3)
        >>> res is c
        True
        >>> # 5. test k greater than k of node and not found
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.right = c
        >>> res = s._get_node(b, 2.5)
        >>> res is None
        True
        """
        if not node:
            return

        if node.k == k:
            return node

        if k < node.k:
            return self._get_node(node.left, k)

        return self._get_node(node.right, k)

    @property
    def min(self):
        """
        >>> # 1. test index error
        >>> s = BSTKVStore()
        >>> s.min
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. test get min
        >>> s = BSTKVStore()
        >>> s[2] = 'b'
        >>> s[1] = 'a'
        >>> s[3] = 'c'
        >>> s.min
        1
        """
        if len(self) == 0:
            raise IndexError('underflow')
        n = self._get_min_node(self.root)
        return n.k

    def _get_min_node(self, node):
        """
        >>> # 1. test node is the min node
        >>> s = BSTKVStore()
        >>> b = Node(1, 'b')
        >>> n = s._get_min_node(b)
        >>> n is b
        True
        >>> # 2. test node is not the min node
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'a')
        >>> b.left = a
        >>> n = s._get_min_node(b)
        >>> n is a
        True
        """
        if node.left is None:
            return node
        return self._get_min_node(node.left)

    def _get_max_node(self, node):
        """
        >>> # 1. test node is the max node
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> n = s._get_max_node(b)
        >>> n is b
        True
        >>> # 2. test node is not the max node
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.right = c
        >>> n = s._get_max_node(b)
        >>> n is c
        True
        """
        if node.right is None:
            return node
        return self._get_max_node(node.right)

    @property
    def max(self):
        """
        >>> # 1. test index error
        >>> s = BSTKVStore()
        >>> s.max
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. test get max
        >>> s = BSTKVStore()
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> s[1] = 'a'
        >>> s.max
        3
        """
        if len(self) == 0:
            raise IndexError('underflow')
        n = self._get_max_node(self.root)
        return n.k

    def delete_min(self):
        """
        >>> # 1. test index error
        >>> s = BSTKVStore()
        >>> s.delete_min()
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. test delete min
        >>> s = BSTKVStore()
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> s[1] = 'a'
        >>> s.min
        1
        >>> s.delete_min()
        >>> s.min
        2
        >>> s.delete_min()
        >>> s.min
        3
        """
        if len(self) == 0:
            raise IndexError('underflow')
        self.root = self._delete_min_node(self.root)

    def _delete_min_node(self, node):
        """
        >>> # 1. node is the min node
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.right = c
        >>> n = s._delete_min_node(b)
        >>> n is c
        True
        >>> # 1. node is not the min node
        >>> s = BSTKVStore()
        >>> a = Node(1, 'a')
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.left = a
        >>> b.right = c
        >>> n = s._delete_min_node(b)
        >>> n is b
        True
        >>> n.left is None
        True
        """
        if node.left is None:
            return node.right
        node.left = self._delete_min_node(node.left)
        node.size = 1 + self._get_size(node.left) + self._get_size(node.right)
        return node

    def _delete_max_node(self, node):
        """
        >>> # 1. node is the max node
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'd')
        >>> b.left = a
        >>> n = s._delete_max_node(b)
        >>> n is a
        True
        >>> # 2. node is not the max node
        >>> s = BSTKVStore()
        >>> a = Node(1, 'a')
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.left = a
        >>> b.right = c
        >>> n = s._delete_max_node(b)
        >>> n is b
        True
        >>> n.right is None
        True
        """
        if node.right is None:
            return node.left
        node.right = self._delete_max_node(node.right)
        node.size = 1 + self._get_size(node.left) + self._get_size(node.right)
        return node

    def delete_max(self):
        """
        >>> # 1. test index error
        >>> s = BSTKVStore()
        >>> s.delete_max()
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> # 2. test delete max
        >>> s = BSTKVStore()
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> s[1] = 'a'
        >>> s.max
        3
        >>> s.delete_max()
        >>> s.max
        2
        >>> s.delete_max()
        >>> s.max
        1
        """
        if len(self) == 0:
            raise IndexError('underflow')
        self.root = self._delete_max_node(self.root)

    def __delitem__(self, k):
        """
        >>> s = BSTKVStore()
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> s[1] = 'a'
        >>> s[3]
        'c'
        >>> del s[3]
        >>> s[3]
        Traceback (most recent call last):
            ...
        KeyError: 3
        """
        self.root = self._delete_node(self.root, k)

    def _delete_node(self, node, k):
        """
        >>> # 1. test node is None
        >>> s = BSTKVStore()
        >>> res = s._delete_node(None, 'k')
        >>> res is None
        True
        >>> # 2. test k < node.k
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'a')
        >>> n = s._delete_node(b, 1)
        >>> n is b
        True
        >>> n.left is None
        True
        >>> # 3. test k > node.k
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> n = s._delete_node(b, 3)
        >>> n is b
        True
        >>> n.right is None
        True
        >>> # 4. test k == node.k
        >>> s = BSTKVStore()
        >>> a = Node(1, 'a')
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.left = a
        >>> b.right = c
        >>> n = s._delete_node(b, 2)
        >>> n is c
        True
        >>> n.left is a
        True
        """
        if node is None:
            return

        if k < node.k:
            node.left = self._delete_node(node.left, k)
        elif k > node.k:
            node.right = self._delete_node(node.right, k)
        else:
            if node.left is None:
                return node.right
            if node.right is None:
                return node.left

            n = self._get_min_node(node.right)
            n.right = self._delete_min_node(node.right)
            n.left = node.left
            node = n

        node.size = 1 + self._get_size(node.left) + self._get_size(node.right)
        return node

    def floor(self, k):
        """
        Return the largest key <= k
        >>> n = BSTKVStore()
        >>> n[1] = 'a'
        >>> n[2] = 'b'
        >>> n[3] = 'c'
        >>> n.floor(2)
        2
        >>> n.floor(4)
        3
        >>> n.floor(1.5)
        1
        >>> n.floor(0.5)

        """
        n = self._get_floor_node(self.root, k)
        return n.k if n else None

    def _get_floor_node(self, node, k):
        """
        Return the largest node with key <= k
        >>> # 1. test node is None
        >>> s = BSTKVStore()
        >>> n = s._get_floor_node(None, 'k')
        >>> n is None
        True
        >>> # 2. test node.k == k
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> n = s._get_floor_node(b, 2)
        >>> n is b
        True
        >>> # 3. test k  < node.k
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'a')
        >>> b.left = a
        >>> n = s._get_floor_node(b, 1.5)
        >>> n is a
        True
        >>> # 4. test k < node.k
        >>> s = BSTKVStore()
        >>> a = Node(1, 'a')
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.left = a
        >>> b.right = c
        >>> n = s._get_floor_node(b, 3)
        >>> n is c
        True
        >>> # 5. test k < node.k and not found in right
        >>> s = BSTKVStore()
        >>> a = Node(1, 'a')
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.left = a
        >>> b.right = c
        >>> n = s._get_floor_node(b, 2.5)
        >>> n is b
        True
        >>> # 6. test k  < node.k and not found in left
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> n = s._get_floor_node(b, 1.5)
        >>> n is None
        True
        """
        if not node:
            return
        if node.k == k:
            return node

        if k < node.k:
            return self._get_floor_node(node.left, k)

        n = self._get_floor_node(node.right, k)
        if not n:
            return node
        return n

    def floor2(self, k):
        """
        Return the largest key <= k
        >>> n = BSTKVStore()
        >>> n[1] = 'a'
        >>> n[2] = 'b'
        >>> n[3] = 'c'
        >>> n.floor(2)
        2
        >>> n.floor(4)
        3
        >>> n.floor(1.5)
        1
        >>> n.floor(0.5)

        """
        return self._get_floor_node2(self.root, k, None)

    def _get_floor2(self, node, k, default):
        if node is None:
            return default
        if node.k == k:
            return k

        if k < node.k:
            return self._get_floor2(node.left, k, default)

        return self._get_floor2(node.right, k, node.k)

    def ceiling(self, k):
        """
        Return the smallest key that <= k
        """
        pass

    def _get_ceiling_node(self, node, k):
        """
        Return the smallest node with key >= k
        >>> # 1. test node is None
        >>> s = BSTKVStore()
        >>> n = s._get_ceiling_node(None, 'k')
        >>> n is None
        True
        >>> # 2. test node.k == k
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> n = s._get_ceiling_node(b, 2)
        >>> n is b
        True
        >>> # 3. test k > node.k and found
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.right = c
        >>> n = s._get_ceiling_node(b, 2.5)
        >>> n is c
        True
        >>> # 4. test k > node.k and not found
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> b.right = c
        >>> n = s._get_ceiling_node(b, 3.5)
        >>> n is None
        True
        >>> # 5. test k < node.k and found
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'a')
        >>> b.left = a
        >>> n = s._get_ceiling_node(b, 0.5)
        >>> n is a
        True
        >>> # 6. test k < node.k and not found
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'a')
        >>> b.left = a
        >>> n = s._get_ceiling_node(b, 1.5)
        >>> n is b
        True
        """
        if node is None:
            return

        if node.k == k:
            return node

        if k > node.k:
            return self._get_ceiling_node(node.right, k)

        n = self._get_ceiling_node(node.left, k)
        if not n:
            return node
        return n

    def index(self, k):
        """
        >>> # 1. test node is None
        >>> s = BSTKVStore()
        >>> s.index(0)
        0
        >>> s[1] = 'a'
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> s.index(1)
        0
        >>> s.index(2)
        1
        >>> s.index(3)
        2
        >>> s.index(4)
        3
        >>> s.index(2.3)
        2
        """
        return self._index(self.root, k)

    def _index(self, node, k):
        """
        >>> # 1. test node is None
        >>> s = BSTKVStore()
        >>> s._index(None, 'k')
        0
        >>> # 2. test k == node.k and node.left is None
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> s._index(b, 2)
        0
        >>> # 3. test k == node.k and node.left is not None
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> a = Node(1, 'a')
        >>> b.left = a
        >>> s._index(b, 2)
        1
        >>> # 4. test k < node.k
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> s._index(b, 1)
        0
        >>> # 5. test k > node.k
        >>> s = BSTKVStore()
        >>> b = Node(2, 'b')
        >>> s._index(b, 3)
        1
        """
        if not node:
            return 0

        if node.k == k:
            return self._get_size(node.left)

        if k < node.k:
            return self._index(node.left, k)

        return self._get_size(node.left) + 1 + self._index(node.right, k)

    def get_by_index(self, i):
        """
        >>> s = BSTKVStore()
        >>> s[1] = 'a'
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> s.get_by_index(0)
        1
        >>> s.get_by_index(1)
        2
        >>> s.get_by_index(2)
        3
        """
        if not 0 <= i < len(self):
            raise IndexError()

        n = self._get_node_by_index(self.root, i)
        return n.k

    def _get_node_by_index(self, node, i):
        """
        >>> # 1. test left_size == i
        >>> s = BSTKVStore()
        >>> a = Node(1, 'a')
        >>> b = Node(2, 'b')
        >>> b.size = 2
        >>> b.left = a
        >>> n = s._get_node_by_index(b, 1)
        >>> n is b
        True
        >>> # 2. test i < left_size
        >>> s = BSTKVStore()
        >>> a = Node(1, 'a')
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> c.size = 3
        >>> b.size = 2
        >>> c.left = b
        >>> b.left = a
        >>> n = s._get_node_by_index(c, 1)
        >>> n is b
        True
        >>> # 3. test i > left_size
        >>> s = BSTKVStore()
        >>> a = Node(1, 'a')
        >>> b = Node(2, 'b')
        >>> c = Node(3, 'c')
        >>> a.right = b
        >>> b.right = c
        >>> b.size = 2
        >>> a.size = 3
        >>> n = s._get_node_by_index(a, 2)
        >>> n is c
        True
        """
        if not node:
            return
        left_size = self._get_size(node.left)

        if left_size == i:
            return node

        if i < left_size:
            return self._get_node_by_index(node.left, i)

        return self._get_node_by_index(node.right, i-left_size-1)

    @property
    def height(self):
        """
        A one-node tree has height of 0
        >>> s = BSTKVStore()
        >>> s.height
        -1
        >>> s[1] = 'a'
        >>> s.height
        0
        >>> s[2] = 'b'
        >>> s.height
        1
        """
        return self._height(self.root)

    def _height(self, node):
        if not node:
            return -1
        return 1 + max(self._height(node.left), self._height(node.right))

    def __contains__(self, k):
        """
        >>> s = BSTKVStore()
        >>> 1 in s
        False
        >>> s[1] = 'a'
        >>> s[2] = 'b'
        >>> 1 in s
        True
        """
        n = self._get_node(self.root, k)
        return n is not None

    def __iter__(self):
        """
        >>> s = BSTKVStore()
        >>> s[1] = 'a'
        >>> s[2] = 'b'
        >>> s[3] = 'c'
        >>> s[4] = 'd'
        >>> for k in s:
        ...    print k
        ...
        1
        2
        3
        4
        """
        q = LinkedQueue()
        q.enqueue(self.root)
        while len(q) > 0:
            n = q.dequeue()
            yield n.k
            if n.left:
                q.enqueue(n.left)
            if n.right:
                q.enqueue(n.right)



def sort(lst):
    """
    >>> lst = [5, 4, 3, 2, 1]
    >>> sort(lst)
    >>> lst
    [1, 2, 3, 4, 5]
    """
    n = len(lst)
    if n <= 1:
        return

    mi = n / 2
    left = lst[:mi]
    right = lst[mi:]
    sort(left)
    sort(right)

    i = j = k = 0
    while i < len(left) and j < len(right):
        if left[i] < right[j]:
            lst[k] = left[i]
            i += 1
        else:
            lst[k] = right[j]
            j += 1
        k += 1

    while i < len(left):
        lst[k] = left[i]
        i += 1
        k += 1

    while j < len(right):
        lst[k] = right[j]
        j += 1
        k += 1



class FixedSizeMaxPriorityQueue(object):

    def __init__(self, capacity=4):
        self.keys = [None] * capacity
        self.n = 0

    def __len__(self):
        """
        >>> q = FixedSizeMaxPriorityQueue()
        >>> len(q)
        0
        >>> q.push(1)
        >>> q.push(2)
        >>> q.push(3)
        >>> len(q)
        3
        >>> q.pop()
        3
        >>> len(q)
        2
        """
        return self.n

    def push(self, i):
        """
        >>> q = FixedSizeMaxPriorityQueue()
        >>> q.push(1)
        >>> q.max
        1
        >>> q.push(3)
        >>> q.max
        3
        >>> q.push(2)
        >>> q.max
        3
        """
        self.n += 1
        self.keys[self.n] = i
        self._swim(self.n)

    def pop(self):
        """
        >>> q = FixedSizeMaxPriorityQueue()
        >>> q.push(1)
        >>> q.push(3)
        >>> q.push(2)
        >>> q.pop()
        3
        >>> q.pop()
        2
        >>> q.pop()
        1
        >>> q.pop()
        Traceback (most recent call last):
            ...
        IndexError: underflow
        """
        if not self.n:
            raise IndexError('underflow')

        keys = self.keys
        res = keys[1]
        keys[1], keys[self.n] = keys[self.n], keys[1]
        keys[self.n] = None
        self.n -= 1

        self._sink(1)
        return res

    @property
    def max(self):
        """
        >>> q = FixedSizeMaxPriorityQueue()
        >>> q.max
        Traceback (most recent call last):
            ...
        IndexError: underflow
        >>> q = FixedSizeMaxPriorityQueue()
        >>> q.keys = [None, 1, 2, 3]
        >>> q.n = 3
        >>> q.max
        1
        """
        if not self.n:
            raise IndexError('underflow')
        return self.keys[1]

    def _swim(self, n):
        """
        >>> q = FixedSizeMaxPriorityQueue()
        >>> q.keys = [None, 1, 2, None, 3]
        >>> q.n = 4
        >>> q._swim(4)
        >>> q.keys
        [None, 3, 1, None, 2]
        >>> q = FixedSizeMaxPriorityQueue()
        >>> q.keys = [None, 2, 1, None, 3]
        >>> q.n = 4
        >>> q._swim(4)
        >>> q.keys
        [None, 3, 2, None, 1]
        """
        keys = self.keys
        while n > 1 and keys[n/2] < keys[n]:
            keys[n/2], keys[n] = keys[n], keys[n/2]
            n /= 2

    def _sink(self, n):
        """
        >>> q = FixedSizeMaxPriorityQueue()
        >>> q.keys = [None, 1, 2, 3]
        >>> q.n = 3
        >>> q._sink(1)
        >>> q.keys
        [None, 3, 2, 1]
        >>> q = FixedSizeMaxPriorityQueue()
        >>> q.keys = [None, 2, 1, 3]
        >>> q.n = 3
        >>> q._sink(1)
        >>> q.keys
        [None, 3, 1, 2]
        """
        keys = self.keys
        while 2 * n <= self.n:
            i = 2 * n
            if i < self.n and keys[i+1] > keys[i]:
                i = i + 1

            if keys[i] <= keys[n]:
                break

            keys[n], keys[i] = keys[i], keys[n]
            n = i



def sort(lst):
    """
    >>> lst = [6, 5, 4, 3, 2]
    >>> sort(lst)
    >>> lst
    [2, 3, 4, 5, 6]
    """
    n = len(lst)
    for slot in range(n-1, 0, -1):
        max_index = 0
        for i in range(1, slot+1):
            if lst[i] > lst[max_index]:
                max_index = i
        lst[max_index], lst[slot] = lst[slot], lst[max_index]



from lib_collection.search.sequential_search_kvstore import (
    SequentialSearchKVStore,
)


class SeparateChainingHashKVStore(object):

    __item_cnt_per_bucket__ = 10

    def __init__(self, d=None, bucket_cnt=4):
        """
        >>> h = SeparateChainingHashKVStore({'c': 3, 'b': 2, 'a': 1})
        >>> h['a']
        1
        >>> h['b']
        2
        >>> h['c']
        3
        """
        self.n = 0
        self.bucket_cnt = bucket_cnt
        self.lst = [SequentialSearchKVStore() for _ in range(self.bucket_cnt)]
        if d:
            s = SeparateChainingHashKVStore(bucket_cnt=bucket_cnt)
            for k, v in d.items():
                s[k] = v
            self.n = s.n
            self.bucket_cnt = s.bucket_cnt
            self.lst = s.lst

    def __repr__(self):
        """
        >>> h = SeparateChainingHashKVStore(bucket_cnt=1)
        >>> h['a'] = 1
        >>> h['b'] = 2
        >>> h['c'] = 3
        >>> h
        SeparateChainingHashKVStore({'c': 3, 'b': 2, 'a': 1})
        """
        s = ', '.join(
            '{}: {}'.format(repr(k), repr(v))
            for k, v in self.items()
        )
        return 'SeparateChainingHashKVStore({%s})' % s

    def __len__(self):
        """
        >>> h = SeparateChainingHashKVStore(bucket_cnt=1)
        >>> len(h)
        0
        >>> h['a'] = 1
        >>> h['b'] = 2
        >>> h['c'] = 3
        >>> len(h)
        3
        >>> del h['c']
        >>> len(h)
        2
        """
        return self.n

    def __iter__(self):
        """
        >>> h = SeparateChainingHashKVStore(bucket_cnt=1)
        >>> h['a'] = 2
        >>> h['b'] = 1
        >>> h['c'] = 3
        >>> for k in h:
        ...     print k
        ...
        c
        b
        a
        """
        for kvstore in self.lst:
            for k in kvstore:
                yield k

    def __contains__(self, k):
        """
        >>> h = SeparateChainingHashKVStore(bucket_cnt=1)
        >>> 'a' in h
        False
        >>> h['a'] = 1
        >>> 'a' in h
        True
        """
        i = self._hash(k)
        d = self.lst[i]
        return k in d

    def items(self):
        """
        >>> h = SeparateChainingHashKVStore(bucket_cnt=1)
        >>> h['a'] = 2
        >>> h['b'] = 1
        >>> h['c'] = 3
        >>> for k, v in h.items():
        ...     print k, v
        ...
        c 3
        b 1
        a 2
        """
        for kvstore in self.lst:
            for k, v in kvstore.items():
                yield k, v

    def __setitem__(self, k, v):
        """
        >>> # 1. test not contains
        >>> h = SeparateChainingHashKVStore(bucket_cnt=1)
        >>> len(h)
        0
        >>> h['a'] = 1
        >>> len(h)
        1
        >>> 'a' in h.lst[0]
        True
        >>> # 2. test contains
        >>> h = SeparateChainingHashKVStore(bucket_cnt=1)
        >>> s = SequentialSearchKVStore()
        >>> s['a'] = 1
        >>> h.n = 1
        >>> h.lst = [s]
        >>> h['a'] = 2
        >>> len(h)
        1
        >>> s['a']
        2
        >>> # 3. test resize up
        >>> h = SeparateChainingHashKVStore(bucket_cnt=4)
        >>> h.n = 40
        >>> h['a'] = 1
        >>> h.bucket_cnt
        8
        """
        if self.n >= 10 * self.bucket_cnt:
            self._resize(self.bucket_cnt*2)

        i = self._hash(k)
        d = self.lst[i]
        if k not in d:
            self.n += 1
        d[k] = v

    def __getitem__(self, k):
        """
        >>> # 1. test contains
        >>> h = SeparateChainingHashKVStore()
        >>> h['a'] = 1
        >>> h['a']
        1
        >>> # 2. test not contains
        >>> h = SeparateChainingHashKVStore()
        >>> h['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        """
        i = self._hash(k)
        d = self.lst[i]
        return d[k]

    def __delitem__(self, k):
        """
        >>> # 1. test contains
        >>> h = SeparateChainingHashKVStore()
        >>> h['a'] = 1
        >>> del h['a']
        >>> h['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        >>> # 2. test not contains
        >>> h = SeparateChainingHashKVStore()
        >>> del h['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        """
        i = self._hash(k)
        d = self.lst[i]
        if k in d:
            self.n -= 1
        del d[k]

    def _hash(self, k):
        """
        >>> d = SeparateChainingHashKVStore()
        >>> d._hash('a')
        1
        >>> d._hash('b')
        2
        >>> d._hash('c')
        3
        >>> d._hash('d')
        0
        """
        return sum(ord(i) for i in str(k)) % self.bucket_cnt

    def _resize(self, bucket_cnt):
        s = SeparateChainingHashKVStore(bucket_cnt=bucket_cnt)
        for k, v in self.items():
            s[k] = v
        self.bucket_cnt = bucket_cnt



def get_binary_tree(r):
    return [r, [], []]


def insert_left(root, subtree):
    left_node = root.pop(1)
    if len(left_node) > 1:
        root.insert(1, [subtree, left_node, []])
    else:
        root.insert(1, [subtree, [], []])
    return root


def insert_right(root, subtree):
    right_node = root.pop(2)
    if len(right_node) > 1:
        root.insert(2, [subtree, [], right_node])
    else:
        root.insert(2, [subtree, [], []])
    return root


def get_root_value(root):
    return root[0]


def set_root_value(root, value):
    root[0] = value


def get_left_child(root):
    return root[1]


def get_right_child(root):
    return root[2]

r = get_binary_tree(3)
insert_left(r, 4)
insert_left(r, 5)
insert_right(r, 6)
insert_right(r, 7)


left = get_left_child(r)
print left

set_root_value(left, 9)
print r

insert_left(left, 11)
print r

print get_right_child(get_right_child(r))


class Tree(object):
    def __init__(self, v):
        self.lst = [v, [], []]

    def __len__(self):
        return 1 + len(self.lst[1]) + len(self.lst[2])

    def insert_left(self, v):
        t = Tree(v)
        if len(self.lst[1]) == 0:
            self.lst[1] = t
            return
        t.insert_left(self.lst[1])
        self.lst[1] = t

    def insert_right(self, v):
        t = Tree(v)
        if len(self.lst[2]) == 0:
            self.lst[2] = t
            return
        t.insert_right(self.lst[2])
        self.lst[2] = t

    @property
    def root_value(self):
        return self.lst[0]

    @root_value.setter
    def root_value(self, v):
        self.lst[0] = v

    @property
    def left_tree(self):
        return self.lst[1] if self.lst[1] else None

    @property
    def right_tree(self):
        return self.lst[2] if self.lst[2] else None

    def __repr__(self):
        return 'Tree({}, {}, {})'.format(self.root_value, self.left_tree, self.right_tree)


r = Tree(3)
r.insert_left(4)
r.insert_left(5)
r.insert_right(6)
r.insert_right(7)
print r.left_tree
r.left_tree.root_value = 9
print r
r.left_tree.insert_left(11)
print r
print r.right_tree.right_tree



from lib_collection.node import Node


class LinkedQueue(object):

    def __init__(self, lst=None):
        """
        >>> q = LinkedQueue(['a', 2, 'c'])
        >>> q.top
        'a'
        >>> q.dequeue()
        'a'
        >>> q.dequeue()
        2
        >>> len(q)
        1
        >>> q.enqueue(4)
        >>> q.enqueue('e')
        >>> q
        LinkedQueue(['c', 4, 'e'])
        """
        self.n = 0
        self.head = None
        self.tail = None

        if lst is not None:
            q = LinkedQueue()
            for e in lst:
                q.enqueue(e)
            self.n = q.n
            self.head = q.head
            self.tail = q.tail

    def __len__(self):
        """
        >>> q = LinkedQueue()
        >>> len(q)
        0
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> len(q)
        2
        >>> q.dequeue()
        'a'
        >>> q.dequeue()
        'b'
        >>> len(q)
        0
        """
        return self.n

    def __contains__(self, i):
        """
        >>> q = LinkedQueue()
        >>> 'a' in q
        False
        >>> q.enqueue('a')
        >>> 'a' in q
        True
        """
        for j in self:
            if i == j:
                return True
        return False

    def __iter__(self):
        """
        >>> q = LinkedQueue()
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.enqueue('c')
        >>> for i in q:
        ...     print i
        ...
        a
        b
        c
        """
        n = self.head
        while n:
            yield n.v
            n = n.next

    def __repr__(self):
        """
        >>> q = LinkedQueue()
        >>> q.enqueue('a')
        >>> q.enqueue(2)
        >>> q.enqueue('c')
        >>> q
        LinkedQueue(['a', 2, 'c'])
        >>> print q
        LinkedQueue(['a', 2, 'c'])
        """
        return 'LinkedQueue([{}])'.format(', '.join(repr(i) for i in self))

    def enqueue(self, i):
        """
        >>> q = LinkedQueue()
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        """
        n = Node(i)
        if self.head is None:
            self.head = n
            self.tail = n
        else:
            self.tail.next = n
            self.tail = n
        self.n += 1

    def dequeue(self):
        """
        >>> q = LinkedQueue()
        >>> q.enqueue('a')
        >>> q.enqueue('b')
        >>> q.enqueue('c')
        >>> q.dequeue()
        'a'
        >>> q.dequeue()
        'b'
        >>> q.dequeue()
        'c'
        >>> q.dequeue()
        Traceback (most recent call last):
            ...
        IndexError: queue underflowed
        """
        if self.head is None:
            raise IndexError('queue underflowed')
        res = self.head.v
        self.head = self.head.next
        self.n -= 1
        return res

    @property
    def top(self):
        """
        >>> q = LinkedQueue()
        >>> q.top
        Traceback (most recent call last):
            ...
        IndexError: queue underflowed
        >>> q.enqueue('a')
        >>> q.top
        'a'
        >>> q.enqueue('b')
        >>> q.top
        'a'
        """
        if self.head is None:
            raise IndexError('queue underflowed')
        return self.head.v



class LinearProbingKVStore(object):

    def __init__(self, d=None, capacity=4):
        """
        >>> d = LinearProbingKVStore({'a': 1, 'b': 2})
        >>> d['a']
        1
        >>> d['b']
        2
        """
        self.capacity = capacity
        self.n = 0
        self.keys = [None] * self.capacity
        self.values = [None] * self.capacity

        if d:
            s = LinearProbingKVStore(capacity=capacity)
            for k, v in d.items():
                s[k] = v
            self.n = s.n
            self.keys = s.keys
            self.values = s.values

    def __repr__(self):
        """
        >>> d = LinearProbingKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> d
        LinearProbingKVStore({'a': 1, 'b': 2})
        """
        s = ', '.join('{}: {}'.format(repr(k), repr(v)) for k, v in self.items())
        return 'LinearProbingKVStore({%s})' % s

    def __len__(self):
        """
        >>> d = LinearProbingKVStore()
        >>> len(d)
        0
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> len(d)
        2
        >>> del d['a']
        >>> len(d)
        1
        """
        return self.n

    def __contains__(self, k):
        """
        >>> d = LinearProbingKVStore()
        >>> 'a' in d
        False
        >>> d['a'] = 1
        >>> d['e'] = 2
        >>> 'a' in d
        True
        """
        h = self._hash(k)
        while self.keys[h] is not None:
            if self.keys[h] == k:
                return True
            h = (h+1) % self.capacity
        return False

    def __setitem__(self, k, v):
        """
        >>> # 1. test not contains
        >>> d = LinearProbingKVStore(capacity=2)
        >>> d['a'] = 1
        >>> d.keys
        [None, 'a']
        >>> # 2. test contains
        >>> d['a'] = 2
        >>> d.keys
        [None, 'a', None, None]
        >>> # 3. test resize up
        >>> d = LinearProbingKVStore(capacity=4)
        >>> d['a'] = 1
        >>> d['b'] = 1
        >>> d['c'] = 1
        >>> d.capacity
        8
        """
        if self.n * 2 >= self.capacity:
            self._resize(self.capacity*2)

        h = self._hash(k)
        while self.keys[h] is not None:
            if self.keys[h] == k:
                self.values[h] = v
                return
            h = (h+1) % self.capacity
        self.keys[h] = k
        self.values[h] = v
        self.n += 1

    def __iter__(self):
        """
        >>> d = LinearProbingKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> for i in d:
        ...     print i
        ...
        a
        b
        """
        for k in self.keys:
            if k is not None:
                yield k

    def items(self):
        """
        >>> d = LinearProbingKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 2
        >>> for k, v in d.items():
        ...     print k, v
        ...
        a 1
        b 2
        """
        for k, v in zip(self.keys, self.values):
            if k is not None:
                yield k, v

    def __getitem__(self, k):
        """
        >>> # 1. test key not exists
        >>> d = LinearProbingKVStore(capacity=2)
        >>> d['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        >>> # 2. test key exists
        >>> d['a'] = 1
        >>> d['a']
        1
        >>> d['b']
        Traceback (most recent call last):
            ...
        KeyError: 'b'
        """
        h = self._hash(k)
        while self.keys[h] is not None:
            if self.keys[h] == k:
                return self.values[h]
            h = (h+1) % self.capacity
        raise KeyError(k)

    def __delitem__(self, k):
        """
        >>> # 1. test not contains
        >>> d = LinearProbingKVStore()
        >>> del d['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        >>> # 2. test contains
        >>> d['a'] = 1
        >>> del d['a']
        >>> d['a']
        Traceback (most recent call last):
            ...
        KeyError: 'a'
        >>> # 3. test rehash
        >>> d = LinearProbingKVStore()
        >>> d._hash('a')
        1
        >>> d._hash('e')
        1
        >>> d['a'] = 1
        >>> d['e'] = 2
        >>> d.keys
        [None, 'a', 'e', None]
        >>> del d['a']
        >>> d.keys
        [None, 'e', None, None]
        """
        h = self._hash(k)
        while True:
            if self.keys[h] is None:
                raise KeyError(k)
            if self.keys[h] == k:
                break
            h = (h+1) % self.capacity

        # delete
        self._delete_kv_at_index(h)

        # rehash the keys in same bucket
        h = (h+1) % self.capacity


        while self.keys[h] is not None:
            self._rehash(h)
            h = (h+1) % self.capacity

        # resize down
        if self.n and self.n * 8 <= self.capacity:
            self._resize(self.capacity/2)

    def _delete_kv_at_index(self, h):
        self.keys[h] = None
        self.values[h] = None
        self.n -= 1

    def _rehash(self, h):
        key = self.keys[h]
        value = self.values[h]
        self._delete_kv_at_index(h)

        self[key] = value

    def _hash(self, k):
        """
        >>> d = LinearProbingKVStore()
        >>> d._hash('a')
        1
        >>> d._hash('b')
        2
        >>> d._hash('c')
        3
        >>> d._hash('d')
        0
        """
        return sum(ord(i) for i in str(k)) % self.capacity

    def _resize(self, capacity):
        """
        >>> d = LinearProbingKVStore()
        >>> d['a'] = 1
        >>> d['b'] = 1
        >>> d.capacity
        4
        >>> d['c'] = 1
        >>> d.capacity
        8
        >>> del d['a']
        >>> del d['b']
        >>> d.capacity
        4
        """
        d = LinearProbingKVStore(capacity=capacity)
        for k, v in self.items():
            d[k] = v
        self.capacity = capacity
        self.keys = d.keys
        self.values = d.values



def sort(lst):
    """
    >>> lst = [5, 4, 3, 2, 1]
    >>> sort(lst)
    >>> lst
    [1, 2, 3, 4, 5]
    """
    _sort(lst, len(lst)-1)


def _sort(lst, slot):
    if slot == 0:
        return
    max_index = slot
    for i in range(slot):
        if lst[i] > lst[max_index]:
            max_index = i
    lst[max_index], lst[slot] = lst[slot], lst[max_index]
    _sort(lst, slot-1)



class ResizingArrayStack(object):
    """
    This implementation uses a resizing array, which double the underlying array
    when it is full and halves the underlying array when it is one-quarter full.
    """

    def __init__(self, lst=None):
        """
        >>> stack = ResizingArrayStack(["a", 1, "b"])
        >>> stack.pop()
        'a'
        >>> stack.push('c')
        >>> stack
        ResizingArrayStack(['c', 1, 'b'])
        """
        self.capacity = 2
        self.resizing_array = [None] * self.capacity
        self.n = 0

        if lst:
            stack = ResizingArrayStack()
            for e in lst[::-1]:
                stack.push(e)

            self.capacity = stack.capacity
            self.resizing_array = stack.resizing_array
            self.n = stack.n

    def __len__(self):
        """
        >>> stack = ResizingArrayStack()
        >>> len(stack)
        0
        >>> stack.push('a')
        >>> len(stack)
        1
        """
        return self.n

    def __contains__(self, i):
        """
        >>> stack = ResizingArrayStack()
        >>> stack.push('a')
        >>> stack.push('b')
        >>> 'a' in stack
        True
        >>> 'c' in stack
        False
        """
        for j in self:
            if j == i:
                return True
        return False

    def __iter__(self):
        """
        >>> stack = ResizingArrayStack()
        >>> stack.push('a')
        >>> stack.push('b')
        >>> for i in stack:
        ...     print i
        ...
        b
        a
        """
        for i in range(self.n-1, -1, -1):
            yield self.resizing_array[i]

    def __repr__(self):
        """
        >>> stack = ResizingArrayStack()
        >>> stack.push('a')
        >>> stack.push('b')
        >>> stack
        ResizingArrayStack(['b', 'a'])
        >>> print stack
        ResizingArrayStack(['b', 'a'])
        """
        return 'ResizingArrayStack([{}])'.format(', '.join(repr(i) for i in self))

    def push(self, i):
        """
        >>> stack = ResizingArrayStack()
        >>> stack.push('a')
        >>> stack.push('b')
        >>> stack.push('c')
        >>> stack
        ResizingArrayStack(['c', 'b', 'a'])
        """
        if self.n == self.capacity:
            self._resize(self.capacity*2)
        self.resizing_array[self.n] = i
        self.n += 1

    def pop(self):
        """
        >>> stack = ResizingArrayStack()
        >>> stack.push('a')
        >>> stack.push('b')
        >>> stack.push('c')
        >>> stack.push('d')
        >>> stack.push('e')
        >>> stack.push('f')
        >>> stack.push('g')
        >>> stack.push('h')
        >>> stack.pop()
        'h'
        >>> stack.pop()
        'g'
        >>> stack.pop()
        'f'
        >>> stack.pop()
        'e'
        >>> stack.pop()
        'd'
        >>> stack.capacity
        8
        >>> stack.pop()
        'c'
        >>> len(stack)
        2
        >>> stack.capacity
        4
        >>> stack.pop()
        'b'
        >>> len(stack)
        1
        >>> stack.capacity
        2
        >>> stack.pop()
        'a'
        >>> len(stack)
        0
        >>> stack.capacity
        2
        >>> stack.pop()
        Traceback (most recent call last):
            ...
        IndexError: pop from empty stack
        """
        if len(self) == 0:
            raise IndexError('pop from empty stack')
        self.n -= 1
        res = self.resizing_array[self.n]
        self.resizing_array[self.n] = None

        if len(self) and len(self) * 4 <= self.capacity:
            self._resize(self.capacity/2)
        return res

    @property
    def top(self):
        """
        >>> stack = ResizingArrayStack()
        >>> stack.top
        Traceback (most recent call last):
            ...
        IndexError: pop from empty stack
        >>> stack.push('a')
        >>> stack.top
        'a'
        >>> stack.push('b')
        >>> stack.top
        'b'
        """
        if len(self) == 0:
            raise IndexError('pop from empty stack')
        return self.resizing_array[self.n-1]

    def _resize(self, m):
        """
        >>> stack = ResizingArrayStack()
        >>> stack.push('a')
        >>> stack.push('b')
        >>> stack._resize(6)
        >>> stack.resizing_array
        ['a', 'b', None, None, None, None]
        >>> stack.capacity
        6
        >>> len(stack)
        2
        >>> stack
        ResizingArrayStack(['b', 'a'])
        """
        resizing_array = [None] * m
        for i in range(self.n):
            resizing_array[i]= self.resizing_array[i]
        self.resizing_array = resizing_array
        self.capacity = m



def sort(lst):
    """
    >>> lst = [5, 4, 3, 2, 1]
    >>> sort(lst)
    >>> lst
    [1, 2, 3, 4, 5]
    """
    n = len(lst)
    # make max priority queue
    for i in range(n/2, 0, -1):
        _sink(lst, i, n)

    # sort
    while n > 1:
        _exchange(lst, 1, n)
        n -= 1
        _sink(lst, 1, n)


def _sink(lst, i, n):
    while 2 * i <= n:
        j = 2 * i
        if j < n and _compare(lst, j, j+1) < 0:
            j += 1
        if _compare(lst, i, j) >= 0:
            break
        _exchange(lst, i, j)
        i = j


def _compare(lst, i, j):
    return lst[i-1] - lst[j-1]


def _exchange(lst, i, j):
    lst[i-1], lst[j-1] = lst[j-1], lst[i-1]



def sort(lst):
    """
    >>> lst = [5, 4, 3, 2, 1]
    >>> sort(lst)
    >>> lst
    [1, 2, 3, 4, 5]
    """
    n = len(lst)
    for i in range(1, n):
        j = i
        v = lst[i]
        while j > 0 and lst[j-1] > v:
            lst[j] = lst[j-1]
            j -= 1
        lst[j] = v



class ResizingArrayQueue(object):

    def __init__(self, lst=None, capacity=2):
        self.capacity = capacity
        self.lst = [None] * self.capacity
        self.head = 0
        self.tail = 0
        self.n = 0

    def __len__(self):
        """
        >>> queue = ResizingArrayQueue()
        >>> len(queue)
        0
        >>> queue.enqueue('a')
        >>> len(queue)
        1
        >>> queue.enqueue('b')
        >>> len(queue)
        2
        """
        return self.n

    def __contains__(self, i):
        """
        >>> queue = ResizingArrayQueue()
        >>> 'a' in queue
        False
        >>> queue.enqueue('a')
        >>> queue.enqueue('b')
        >>> 'a' in queue
        True
        """
        for j in self:
            if j == i:
                return True
        return False

    def __iter__(self):
        """
        >>> queue = ResizingArrayQueue()
        >>> queue.enqueue('a')
        >>> queue.enqueue('b')
        >>> for i in queue:
        ...     print i
        ...
        a
        b
        """
        n = self.head
        for _ in range(len(self)):
            if n == self.capacity:
                n = 0
            yield self.lst[n]
            n += 1

    def __repr__(self):
        """
        >>> queue = ResizingArrayQueue()
        >>> queue.enqueue('a')
        >>> queue.enqueue('b')
        >>> queue
        ResizingArrayQueue(['a', 'b'])
        >>> print queue
        ResizingArrayQueue(['a', 'b'])
        """
        return 'ResizingArrayQueue([{}])'.format(', '.join(repr(i) for i in self))

    def enqueue(self, i):
        """
        >>> queue = ResizingArrayQueue()
        >>> queue.enqueue('a')
        >>> queue.enqueue('b')
        >>> queue.enqueue('c')
        >>> queue
        ResizingArrayQueue(['a', 'b', 'c'])
        >>> queue.capacity
        4
        """
        if len(self) == self.capacity:
            self._resize(self.capacity*2)

        if self.tail == self.capacity:
            self.tail = 0

        self.lst[self.tail] = i
        self.tail += 1
        self.n += 1

    def dequeue(self):
        """
        >>> queue = ResizingArrayQueue()
        >>> queue.dequeue()
        Traceback (most recent call last):
            ...
        IndexError: dequeue from empty queue
        >>> queue.enqueue('a')
        >>> queue.enqueue('b')
        >>> queue.enqueue('c')
        >>> queue.dequeue()
        'a'
        >>> queue.dequeue()
        'b'
        >>> queue.enqueue('d')
        >>> queue.enqueue('e')
        >>> queue.enqueue('f')
        >>> queue.lst
        ['e', 'f', 'c', 'd']
        >>> queue.enqueue('g')
        >>> queue.capacity
        8
        >>> queue.dequeue()
        'c'
        >>> queue.dequeue()
        'd'
        >>> queue.dequeue()
        'e'
        >>> queue.dequeue()
        'f'
        >>> queue.capacity
        4
        >>> queue.dequeue()
        'g'
        >>> queue.capacity
        2
        """
        if len(self) == 0:
            raise IndexError('dequeue from empty queue')

        if len(self) * 4 <= self.capacity:
            self._resize(self.capacity/2)

        if self.head == self.capacity:
            self.head = 0

        res = self.lst[self.head]
        self.head += 1
        self.n -= 1
        return res

    @property
    def top(self):
        """
        >>> queue = ResizingArrayQueue()
        >>> queue.top
        Traceback (most recent call last):
            ...
        IndexError: top from empty queue
        >>> queue.enqueue('a')
        >>> queue.top
        'a'
        >>> queue.enqueue('b')
        >>> queue.top
        'a'
        >>> queue.dequeue()
        'a'
        >>> queue.top
        'b'
        """
        if len(self) == 0:
            raise IndexError('top from empty queue')
        return self.lst[self.head]

    def _resize(self, n):
        q = ResizingArrayQueue(capacity=n)
        for e in self:
            q.enqueue(e)

        self.capacity = q.capacity
        self.lst = q.lst
        self.head = q.head
        self.tail = q.tail
        self.n = q.n



class Node(object):
    def __init__(self, v):
        self.v = v
        self.next = None

    def __str__(self):
        """
        >>> n = Node('a')
        >>> n
        Node('a')
        >>> n = Node(1)
        >>> n
        Node(1)
        """
        return 'Node({})'.format(repr(self.v))

    __repr__ = __str__



def sort(lst):
    """
    >>> lst = [5, 4, 3, 2, 1]
    >>> sort(lst)
    >>> lst
    [1, 2, 3, 4, 5]
    """
    sublist_count = len(lst) / 2
    while sublist_count > 0:
        for i in range(sublist_count):
            gap_insertion_sort(lst, i, sublist_count)
        sublist_count /= 2


def gap_insertion_sort(lst, start, gap):
    for i in range(start+gap, len(lst), gap):
        v = lst[i]
        j = i
        while j >= gap and lst[j-gap] > v:
            lst[j] = lst[j-gap]
            j -= gap
        lst[j] = v
